<?php
/**
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */

/**
 * Include the {@link shared.make_timestamp.php} plugin
 */
require_once $smarty->_get_plugin_filepath('shared','make_timestamp');
/**
 * Smarty date_format modifier plugin
 *
 * Type:     modifier<br>
 * Name:     date_format<br>
 * Purpose:  format datestamps via strftime<br>
 * Input:<br>
 *         - string: input date string
 *         - format: strftime format for output
 *         - default_date: default date if $string is empty
 * @link http://smarty.php.net/manual/en/language.modifier.date.format.php
 *          date_format (Smarty online manual)
 * @author   Monte Ohrt <monte at ohrt dot com>
 * @param string
 * @param string
 * @param string
 * @return string|void
 * @uses smarty_make_timestamp()
 */
function smarty_modifier_date_format($string, $format="%b %e, %Y", $default_date=null)
{
    if (substr(PHP_OS,0,3) == 'WIN') {
        $hours = strftime('%I', $string);
        $short_hours = ( $hours < 10 ) ? substr( $hours, -1) : $hours; 
        $_win_from = array ('%e',  '%T',       '%D',        '%l');
        $_win_to   = array ('%d', '%H:%M:%S', '%m/%d/%y',  $short_hours);
        $format = str_replace($_win_from, $_win_to, $format);
    }

	if ($string != '') {
        $timestamp = smarty_make_timestamp($string);
    } elseif (isset($default_date) && $default_date != '') {
        $timestamp = smarty_make_timestamp($default_date);
    } else {
        $timestamp = TIME;
    }

	$date = getdate($timestamp);
	$m = $date['mon'];
	$d = $date['mday'];
	$y = $date['year'];
	$wn = 0; // FIXME!!! get weeknumber
	$w = $date['wday'];
	$hr = $date['hours'];
	$pm = ($hr >= 12);
	$ir = ($pm) ? ($hr - 12) : $hr;
	$dy = $date['yday'];
	if ($ir == 0) {
		$ir = 12;
	}
	$min = $date['minutes'];
	$sec = $date['seconds'];

	$s['%a'] = fn_get_lang_var('weekday_abr_'.$w); // abbreviated weekday name
	$s['%A'] = fn_get_lang_var('weekday_'.$w); // full weekday name
	$s['%b'] = fn_get_lang_var('month_name_abr_'.$m); // abbreviated month name
	$s['%B'] = fn_get_lang_var('month_name_'.$m); // full month name
	$s['%c'] = ""; // !!!FIXME: preferred date and time representation for the current locale
	$s['%C'] = 1 + floor($y / 100); // the century number
	$s['%d'] = ($d < 10) ? ("0" . $d) : $d; // the day of the month (range 01 to 31)
	$s['%e'] = $d; // the day of the month (range 1 to 31)
	$s['%'] = $s['%b'];
	$s['%H'] = ($hr < 10) ? ("0" . $hr) : $hr; // hour, range 00 to 23 (24h format)
	$s['%I'] = ($ir < 10) ? ("0" . $ir) : $ir; // hour, range 01 to 12 (12h format)
	$s['%j'] = ($dy < 100) ? (($dy < 10) ? ("00" . $dy) : ("0" . $dy)) : $dy; // day of the year (range 001 to 366)
	$s['%k'] = $hr;		// hour, range 0 to 23 (24h format)
	$s['%l'] = $ir;		// hour, range 1 to 12 (12h format)
	$s['%m'] = ($m < 9) ? ("0" . $m) : $m; // month, range 01 to 12
	$s['%M'] = ($min < 10) ? ("0" . $min) : $min; // minute, range 00 to 59
	$s['%n'] = "\n";		// a newline character
	$s['%p'] = $pm ? "PM" : "AM";
	$s['%P'] = $pm ? "pm" : "am";
	$s['%s'] = floor($timestamp / 1000);
	$s['%S'] = ($sec < 10) ? ("0" . $sec) : $sec; // seconds, range 00 to 59
	$s['%t'] = "\t";		// a tab character
	$s['%T'] = $s['%H'] .':'. $s['%M'] .':'. $s['%S'];
	$s['%U'] = $s['%W'] = $s['%V'] = ($wn < 10) ? ("0" . $wn) : $wn;
	$s['%u'] = $w + 1;	// the day of the week (range 1 to 7, 1 = MON)
	$s['%w'] = $w;		// the day of the week (range 0 to 6, 0 = SUN)
	$s['%y'] = substr($y, 2, 2); // year without the century (range 00 to 99)
	$s['%Y'] = $y;		// year with the century
	$s['%%'] = "%";		// a literal '%' character
	$s['%D'] = $s['%m'] .'/'. $s['%d'] .'/'. $s['%y'];// american date style: %m/%d/%y
	// FIXME: %x : preferred date representation for the current locale without the time
	// FIXME: %X : preferred time representation for the current locale without the date
	// FIXME: %G, %g (man strftime)
	// FIXME: %r : the time in am/pm notation %I:%M:%S %p
	// FIXME: %R : the time in 24-hour notation %H:%M
	return preg_replace("/(%.)/e", "\$s['\\1']", $format);
}

/* vim: set expandtab: */

?>
