//
// $Id: chat.js,v 1.1.1.1 2007/12/13 10:07:59 pshevtsov Exp $
//

// Chat object
live_help_chat = Object.extend(live_help, {
	
	MESSAGE_VISITOR: 0,
	MESSAGE_OPERATOR: 1,
	MESSAGE_SYSTEM: 2,

	// Flag: allow/disallow send message
	allow_send: false,

	// Flag: allow operator notification when typing the message
	allow_type_notify: false,

	// Int: ID of session
	session_id: null,

	// Int: ID of operator
	operator_id: null,

	// Int: ID of the latest message
	last_message: 0,

	//
	// Constructor
	//
	init: function(params)
	{
		if (params) {
			for (k in params) {
				this[k] = params[k];
			}
		}

		return (this.session_data) ? this.enter_chat(this.session_data) : true;
	},

	//
	// Enter the chat
	//
	enter_chat: function (force)
	{
		if (document.getElementById('enter_chat_submit').disabled == true) {
			return false;
		}

		if (typeof(force) == 'undefined') {
			force = false;
		}
		
		if (!force) {
			var auth_name_input = document.getElementById('auth_name');
			this.user_name = auth_name_input.value;
			if (!this.user_name) {
				alert(lh_lang.please_input_name);
				auth_name_input.focus();
				return false;
			}

			fn_switch_elms(['auth_name', 'enter_chat_submit'], 'disabled', true);

			var today = new Date();
			data = {
				user_name: this.user_name,
				local_time: Math.round(today.getTime() / 1000)
			}
			data[this.mode_name] = 'request_chat';

			this.request(data, 'GET', 'on_scan_messages');


			fn_switch_elms(['message_text', 'send_button'], 'disabled', true);
			this.operator_name = '';
			this.operator_id = 0;
		} else {
			// On starting chat by invitation
			this.session_id = force.session_id;
			this.user_name = force.user_name;
			this.operator_id = force.operator_id;
			this.operator_name = force.operator_name;
			this.last_message = force.last_message;
			this.show_message(lh_lang.chat_session_started, this.MESSAGE_SYSTEM);
		}

		fn_show_tag('enter', true); // hide
		fn_show_tag('chat', false); // show

		this.start_timer('live_help_chat.scan_messages()');
		this.allow_type_notify = true;
		this.allow_send = true;
	},

	//
	// Event: on-typing notifier
	//
	on_typing: function(evt, text)
	{
		this.text = text;

		var char_code = (evt.which) ? evt.which : evt.keyCode;
		if (char_code == 13) {
			this.send_message();
		}

		if (this.allow_type_notify == false) {
			return false;
		}

		this.allow_type_notify = false;

		setTimeout("live_help_chat.switch_type_notify()", this.request_interval);

		data = {
			'session_id': this.session_id,
			'operator_id': this.operator_id
		};
		data[this.mode_name] = 'type_notify';


		this.request(data, 'GET');
	},

	//
	// On-typing notifier helper
	//
	switch_type_notify: function()
	{
		this.allow_type_notify = !this.allow_type_notify;
	},

	//
	// Open url in new window
	//
	open_url: function(link)
	{
		link = link.replace(/\\\\/g, "\\\\");
		var win = open(link);
		win.focus();
	},

	//
	// Scan for new messages
	//
	scan_messages: function()
	{
		var data = {
			session_id: this.session_id,
			last_message: this.last_message,
			operator_id: this.operator_id
		};
		data[this.mode_name] = 'scan_messages';

		this.request(data, 'GET', 'on_scan_messages');

		this.start_timer('live_help_chat.scan_messages()');
	},

	//
	// Callback: on-scan messages handler
	//
	on_scan_messages: function(js, plain)
	{
		var data = this.parse_xml(plain);
		if (!data) {
			return false;
		}

		var session_data = data.getElementsByTagName('session_data');
		if (session_data.length) {
			this.session_id = parseInt(this.get_subnode_value(session_data[0], 'session_id'));
			return true;
		}

		// Checking operator's status
		var operator_info = data.getElementsByTagName('op');
		if (operator_info.length) {
			var val = this.get_subnode_value(operator_info[0], 'id');
			new_operator = (val != '-1') ? parseInt(val) : -1;
			
			if (new_operator != this.operator_id) {
				this.operator_id = new_operator;
				
				if (this.operator_id != -1) {
					fn_switch_elms(['message_text', 'send_button'], 'disabled', false);
					this.operator_name = this.get_subnode_value(operator_info[0], 'name');
					this.show_message(lh_lang.chat_operator_joined.replace(/\[operator\]/g, this.operator_name), this.MESSAGE_SYSTEM);
				} else {
					this.operator_id = 0;
					fn_switch_elms(['message_text', 'send_button'], 'disabled', true);
					this.show_message(lh_lang.chat_operator_left, this.MESSAGE_SYSTEM);
					fn_show_tag('type_notify', false);
				}
			}
		}

		// Checking typing notification
		if (data.getElementsByTagName('type_on').length) {
			fn_show_tag('type_notify', false);
		} else if (data.getElementsByTagName('type_off').length) {
			fn_show_tag('type_notify', true);
		}

		// Recieving messages
		var messages = data.getElementsByTagName('message');
		for (var i = 0, len = messages.length; i < len; i++) {
			var message_id = this.get_subnode_value(messages[i], 'id');
			/*if (message_id <= this.last_message) { // FIXME? Don't work if chat is requested by visitor
				continue;
			}*/
			this.last_message = message_id;

			var direction = parseInt(this.get_subnode_value(messages[i], 'dir'));
			var message_text = this.get_subnode_value(messages[i], 'text');
			
			// Operator had sent the file
			if (typeof(message_text) != 'string') {
				if (message_text[0] == 'file') {
					var file_name = message_text[1];
					var short_file_name = decodeURI(file_name.substr(file_name.search('_') + 1));
					var download_url = this.request_url + '&' + this.mode_name + '=get_file&file_name=' + file_name;
					if (browser_info.browser == "Internet Explorer") {
						download_url += "&encode_name=1"
					}
					var message_text = lh_lang.chat_recieved_file.replace('[name]', short_file_name).replace('[url]', download_url);
					this.show_message(message_text, this.MESSAGE_SYSTEM);
				}
				if (message_text[0] == 'url') {
					var html_url = "<a href='" + message_text[1] + "' target='_blank'>" + message_text[1] + "</a>";
					if (direction == this.MESSAGE_OPERATOR) {
						html_url += " (" + lh_lang.chat_opened_link + ")";
						setTimeout("live_help_chat.open_url('" + message_text[1] + "')", 1000);
					}
					this.show_message(html_url, direction);
				}
			} else {
				if (direction == 1) {
					// message from operator
					this.show_message(message_text, this.MESSAGE_OPERATOR);
					fn_show_tag('type_notify', true);
				} else {
					// self message
					this.show_message(message_text, this.MESSAGE_VISITOR);
				}
			}
		}
		
		// Actions after recieving message(s)
		if (messages.length) {
			window.focus();
			if (document.getElementById('message_text').disabled == false) {
				document.getElementById('message_text').focus();
			}
		}

		this.allow_send = true;
	},

	//
	// Display message
	//
	show_message: function(text, message_type)
	{
		if (message_type == this.MESSAGE_SYSTEM) {
			text = "<span class=\"system_message\">" + text + "</span><br />";
		} else if (message_type == this.MESSAGE_VISITOR) {
			text = "<span class=\"operator_message\"><span class=\"name\">" + this.user_name + "</span>: " + text + "</span><br />";
		} else if (message_type == this.MESSAGE_OPERATOR) {
			text = "<span class=\"operator_message\"><span class=\"name\">" + this.operator_name + "</span>: " + text + "</span><br />";
		}
		
		document.getElementById('messages').innerHTML += text;
		document.getElementById('messages').scrollTop = document.getElementById('messages').scrollHeight;

		return true;
	},

	//
	// Send message
	//
	send_message: function()
	{

		if (document.getElementById('send_button').disabled == true) {
			return false;
		}

		if (this.allow_send == false) {
			return false;
		}
			
		document.getElementById('message_text').value = '';
		this.allow_send = false;
			
		data = {
			message_data: {
				text: this.text,
				direction: 0,
				from_id: this.session_id,
				to_id: this.operator_id
			}
		};
		data[this.mode_name] = 'send_message';

		this.request(data, 'POST', 'scan_messages');

		return true;
	}
});
